package org.rcisoft.business.device.report.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.greenpineyu.fel.FelEngine;
import com.greenpineyu.fel.FelEngineImpl;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import org.rcisoft.business.device.report.dao.FormulaRepository;
import org.rcisoft.business.device.report.dao.TotalDataRepository;
import org.rcisoft.business.device.report.entity.Formula;
import org.rcisoft.business.device.report.entity.TotalData;
import org.rcisoft.business.device.report.entity.Variable;
import org.rcisoft.business.device.report.service.FormulaService;
import org.rcisoft.business.device.report.service.VariableService;
import org.rcisoft.business.device.report.vo.FormulaVo;
import org.rcisoft.core.util.UuidUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * Created by JiChao on 2018/8/31.
 */
@Service
public class FormulaServiceImpl implements FormulaService {

    @Autowired
    private FormulaRepository formulaRepository;
    @Autowired
    private VariableService variableServiceImpl;
    @Autowired
    private TotalDataRepository totalDataRepository;


    @Override
    public Integer addFormula(Formula formula) {
        formula.setCreateTime(new Date());
        return formulaRepository.addFormula(formula);
    }

    @Override
    public Integer updateFormula(Formula formula) {
        return formulaRepository.updateFormula(formula);
    }

    @Override
    public Integer deleteFormula(String id) {
        return formulaRepository.deleteFormula(id);
    }

    @Override
    public Formula selectFormula(String id) {
        return formulaRepository.selectFormula(id);
    }

    @Override
    public List<Formula> selectFormulaList(String proId) {
        return formulaRepository.selectFormulaList(proId);
    }

    @Transactional
    @Override
    public Integer addFormulaVariable(FormulaVo formulaVo) {
        Formula formula = formulaVo.getFormula();
        String uuid = UuidUtil.create32();
        formula.setId(uuid);
        Integer i = this.addFormula(formula);
        Integer k = variableServiceImpl.addVariable(formulaVo.getVariableList(), uuid, formula.getProId());
        return i;
    }

    @Transactional
    @Override
    public Integer updateFormulaVariable(FormulaVo formulaVo) {
        Formula formula = formulaVo.getFormula();
        String id = formula.getId();
        Integer i = this.updateFormula(formula);
        Integer j = variableServiceImpl.deleteVariable(id);
        Integer k = variableServiceImpl.addVariable(formulaVo.getVariableList(), id, formula.getProId());
        return i;
    }

    @Transactional
    @Override
    public Integer deleteFormulaVariable(String formulaId) {
        Integer i = this.deleteFormula(formulaId);
        Integer j = variableServiceImpl.deleteVariable(formulaId);
        return i;
    }

    @Transactional
    @Override
    public FormulaVo selectFormulaVariable(String formulaId) {
        Formula formula = this.selectFormula(formulaId);
        List<Variable> variableList = variableServiceImpl.selectVariable(formulaId);
        return new FormulaVo(formula, variableList);
    }

    @Override
    public void downloadFormula(HttpServletResponse response, String formulaId, String start, String end) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        // 公式计算
        FelEngine fel = new FelEngineImpl();
        // 根据id查找公式
        Formula formula = this.selectFormula(formulaId);
        // 根据id查找变量
        List<Variable> variableList = variableServiceImpl.selectVariable(formulaId);
        // 根据proId和时间区间，查找数据list
        String proId = formula.getProId();
        start = start + " 00:00:00";
        end = end + " 23:59:59";
        List<TotalData> totalDataList = totalDataRepository.selectTotalData(proId, start, end);
        // 生成excel
        HSSFWorkbook workbook = new HSSFWorkbook();
        //定义sheet
        HSSFSheet sheet = workbook.createSheet("sheet1");
        // 创建第一行
        int i = 0;
        HSSFRow head = sheet.createRow(i++);
        head.createCell(0, CellType.STRING).setCellValue("时间");
        head.createCell(1, CellType.STRING).setCellValue("数值");
        // 循环数据list
        for (TotalData totalData : totalDataList) {
            // 创建row
            HSSFRow row = sheet.createRow(i++);
            // 公式
            String f = formula.getFormula();
            // 得到原始数据
            String json = totalData.getJson();
            // 变成json对象
            JSONObject jsonObject = JSON.parseObject(json);
            // 循环变量list
            for (Variable variable : variableList) {
                // 公式中的变量
                String v = variable.getVariable();
                // 来源
                String s = variable.getSource();
                // 参数
                String p = variable.getParam();
                // 根据来源参数，获取json中对应的值
                String value = null;
                JSONObject s_json = (JSONObject) jsonObject.get(s);
                if (s_json != null) {
                    JSONObject r_json = (JSONObject) s_json.get("REG_VAL");
                    if (r_json != null) {
                        Object p_json = r_json.get(p);
                        if (p_json != null) {
                            value = p_json.toString();
                        }
                    }
                }
                if (value != null)
                    // 替换公式
                    f = f.replaceAll(v, value);
            }
            // 判断表达式是否由数字和运算符号组成
            boolean flag = true;
            String[] split = f.split("");
            for (String s : split) {
                if (!s.matches("\\d|\\(|\\)|\\+|\\-|\\*|/|%|\\."))
                    flag = false;
            }
            String eval = "";
            // 计算出公式的最终结果
            if (flag) {
                Object result = fel.eval(f);
                if (result != null) {
                    eval = result.toString();
                }
            }
//            String eval = fel.eval(f).toString();
            // 放入row中
            row.createCell(0, CellType.STRING).setCellValue(sdf.format(totalData.getTm()));
            row.createCell(1, CellType.STRING).setCellValue(eval);
        }
        //下载
        try(OutputStream outputStream = response.getOutputStream()) {
            //设置下载头
            response.setHeader("Content-disposition", "attachment;filename=" + formulaId + ".xls");
            workbook.write(outputStream);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                workbook.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
